#include "common.hpp"
#include "array.hpp"
#include "action.hpp"

using namespace std::complex_literals;

template <typename Scalar, typename Shape>
void dump(Array<Scalar,Shape> Q)
{
    for (size_t i = 0; i < Q.template shape<0>(); ++i) {
        for (size_t j = 0; j < Q.template shape<1>(); ++j) {
            for (size_t k = 0; k < Q.template shape<2>(); ++k) {
                std::cout << i << "," << j << "," << k << " = " << Q(i,j,k) << std::endl;
            }
        }
    }

    for (size_t i = 0; i < Q.data().size(); ++i) {
        std::cout << Q.data()[i] << std::endl;
    }
}

int main()
{
    std::vector<ADComplex> v(2*2*2);
    Array<ADComplex,Shape<Dynamic,2,2> > Q0(v, {2,2,2});
    auto Q = Q0.part(1);

    std::cout << Q0.index(1u,0u,0u) << std::endl;

    S_2(Q);

    std::cout << Q.index(0u,0u) << std::endl;

    auto mat = Q.to_matrix();

    mat = mat * mat * mat * adcomplex(1j);
    mat(0,0) = mat(1,1) = mat.trace();

    dump(Q0);
    std::cout << mat << std::endl;

    static_assert(std::is_same<decltype(mat), Eigen::Map<Eigen::Matrix<ADComplex, 2, 2, Eigen::RowMajor> > >::value, "wrong type");

    return 0;
}
